﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Diagnostics;

namespace ShowHot.ComponentOrganizing
{
    public partial class DatePickerControl : UserControl
    {
        #region Members

        public enum RunningStates
        {
            Stop = 0,
            Forward = 1,
            Backward = 2
        }

        /// <summary>
        /// 날짜변경 (일반, 일주일 이동요청)
        /// </summary>
        private RunningStates _runningState = RunningStates.Stop;

        //선택된 날짜

        public DateTime CurrentDate
        {
            get
            {
                return _currentDate;
            }
            set
            {
                // 날짜를 비교합니다.
                if (_currentDate == value)
                    return;

                _currentDate = value;

                setDateComponent(_currentDate);

                // 이벤트 발생
                OnDateChanged();
            }
        }
        private DateTime _currentDate = DateTime.Now;

        // 내부적으로 애니메이션 등을 소화하기 위한 것
        private DateTime _currentDateInner = DateTime.Now;

        /// <summary>
        /// 잠금
        /// </summary>
        private bool _lock = false;
        
        #endregion Members


        #region Events

        /// <summary>
        /// 날짜가 변경되었을 때 발생하는 이벤트
        /// </summary>
        public event EventHandler DateChanged;

        #endregion Events


        #region Constructors

        /// <summary>
        /// 기본생성자
        /// </summary>
        public DatePickerControl()
        {
            InitializeComponent();

            //초기상태 설정
            InitStates();

            //컴포넌트 들에 날짜를 입력
            setDateComponent(_currentDate);

            //클릭이벤트를 부여함
            LinkEvents();
        }

        #endregion Constructors


        /// <summary>
        /// 날짜를 올림
        /// </summary>
        private void GoNextDay()
        {
            if (_runningState != RunningStates.Stop)
                _runningCount++;

            foreach (DatePickerComponent c in layoutDate.Children)
            {
                //스테이트가 가장 위 이면 가장 아래로 내려라
                if (c.State == DatePickerComponent.ComponentStates.HideUp)
                {
                    c.State = DatePickerComponent.ComponentStates.HideDown;
                    continue;
                }

                c.State--;
            }

            _currentDateInner = _currentDateInner.AddDays(1);
            ChangeEvents();
            setDateComponent(_currentDateInner);
        }
        #region Methods

        /// <summary>
        /// 날짜를 내림
        /// </summary>
        private void GoPreviousDay()
        {
            if (_runningState != RunningStates.Stop)
                _runningCount++;

            foreach (DatePickerComponent c in layoutDate.Children)
            {
                //스테이트가 가장 아래 이면 가장 위로 올려라
                if (c.State == DatePickerComponent.ComponentStates.HideDown)
                {
                    c.State = DatePickerComponent.ComponentStates.HideUp;
                    continue;
                }

                c.State++;
            }

            _currentDateInner = _currentDateInner.AddDays(-1);
            ChangeEvents();
            setDateComponent(_currentDateInner);
        }

        /// <summary>
        /// 현재 날짜에 날짜를 더해줌
        /// </summary>
        /// <param name="days"></param>
        public void MoveDays(int days)
        {
            if (days == 0)
                return;

            if (_lock == true)
                return;

            if (days > 0)
            {
                GoSomeDayLater(days);
            }
            else
            {
                GoSomeDayAgo(days);
            }
        }

        /// <summary>
        /// 컨포넌트들의 날짜를 설정해줌
        /// </summary>
        private void setDateComponent(DateTime date)
        {
            foreach (DatePickerComponent c in layoutDate.Children)
            {
                switch (c.State)
                {
                    case DatePickerComponent.ComponentStates.HideUp:
                        c.Date = date.AddDays(-2);
                        break;
                    case DatePickerComponent.ComponentStates.ExtraUp:
                        c.Date = date.AddDays(-1);
                        break;
                    case DatePickerComponent.ComponentStates.Main:
                        c.Date = date;
                        break;
                    case DatePickerComponent.ComponentStates.ExtraDown:
                        c.Date = date.AddDays(1);
                        break;
                    case DatePickerComponent.ComponentStates.HideDown:
                        c.Date = date.AddDays(2);
                        break;
                    default:
                        break;
                }
            }
        }

        /// <summary>
        /// 클릭이벤트를 부여
        /// </summary>
        private void ChangeEvents()
        {
            foreach (DatePickerComponent c in layoutDate.Children)
            {
                c.MouseLeftButtonDown -= new MouseButtonEventHandler(Extra_upButtonDown);
                c.MouseLeftButtonDown -= new MouseButtonEventHandler(Extra_downButtonDown);
            }

            LinkBasicEvents();
        }

        private void LinkBasicEvents()
        {
            foreach (DatePickerComponent c in layoutDate.Children)
            {
                switch (c.State)
                {
                    case DatePickerComponent.ComponentStates.HideUp:
                        break;
                    case DatePickerComponent.ComponentStates.ExtraUp:
                        c.MouseLeftButtonDown += new MouseButtonEventHandler(Extra_upButtonDown);
                        break;
                    case DatePickerComponent.ComponentStates.Main:
                        break;
                    case DatePickerComponent.ComponentStates.ExtraDown:
                        c.MouseLeftButtonDown += new MouseButtonEventHandler(Extra_downButtonDown);
                        break;
                    case DatePickerComponent.ComponentStates.HideDown:
                        break;
                    default:
                        break;
                }
            }
        }

        private void LinkEvents()
        {
            LinkBasicEvents();
            (layoutDate.Children[0] as DatePickerComponent).MotionFinished += new EventHandler(DatePickerControl_MotionFinished);
        }

        /// <summary>
        /// 일주일 후로 이동합니다.
        /// </summary>
        public void GoSomeDayLater(int days)
        {
            if (_lock == true)
                return;

            // 업데이트
            CurrentDate = _currentDate.AddDays(days);

            PreRun(days);

            _runningState = RunningStates.Forward;
            GoNextDay();
        }

        /// <summary>
        /// 일주일 전으로 이동합니다.
        /// </summary>
        public void GoSomeDayAgo(int days)
        {
            if (_lock == true)
                return;

            // 업데이트
            CurrentDate = _currentDate.AddDays(days);
            
            PreRun(days);
            _runningState = RunningStates.Backward;
            GoPreviousDay();
        }

        private void PreRun(int days)
        {
            _lock = true;

            LayoutRoot.IsHitTestVisible = false;

            _runningTargetDays = Math.Abs(days);
            _runningCount = 0;
        }

        private int _runningCount = 0;
        private int _runningTargetDays = 7;
        void DatePickerControl_MotionFinished(object sender, EventArgs e)
        {
            if (_runningState == RunningStates.Stop)
            {
                _runningCount = 0;
                return;
            }

            // 일주일 뛰었으면 그만~
            if (_runningCount == _runningTargetDays &&
                _runningState != RunningStates.Stop)
            {
                PostRun();
                return;
            }

            switch (_runningState)
            {
                case RunningStates.Stop:
                    break;
                case RunningStates.Forward:
                    GoNextDay();
                    break;
                case RunningStates.Backward:
                    GoPreviousDay();
                    break;
                default:
                    break;
            }
        }

        private void PostRun()
        {
            _lock = false;

            LayoutRoot.IsHitTestVisible = true;

            _runningCount = 0;
            _runningState = RunningStates.Stop;
        }

        /// <summary>
        /// 컴포넌트들의 초기화
        /// </summary>
        private void InitStates()
        {
            dateComponent1.State = DatePickerComponent.ComponentStates.HideUp;
            dateComponent2.State = DatePickerComponent.ComponentStates.ExtraUp;
            dateComponent3.State = DatePickerComponent.ComponentStates.Main;
            dateComponent4.State = DatePickerComponent.ComponentStates.ExtraDown;
            dateComponent5.State = DatePickerComponent.ComponentStates.HideDown;

            // 메인을 오늘 날짜로 설정
            dateComponent3.Date = DateTime.Now;

            _currentDateInner = dateComponent3.Date;
            _currentDate = _currentDateInner;
        }

        #endregion Methods

        #region EventHandlers

        void Extra_downButtonDown(object sender, MouseButtonEventArgs e)
        {
            GoNextDay();

            // 1일 증가
            CurrentDate = _currentDate.AddDays(1);
        }

        void Extra_upButtonDown(object sender, MouseButtonEventArgs e)
        {
            GoPreviousDay();

            // 1일 감소
            CurrentDate = _currentDate.AddDays(-1);
        }

        #endregion EventHandlers

        #region EventFires

        private void OnDateChanged()
        {
            if (DateChanged == null)
                return;

            DateChanged(this, null);
        }

        #endregion EventFires

    }
}
